function[Ft,dFdu,broken] = Truss2D(x,Ldes)
%CB_2dtruss_CT_p - Truss force and consistent tangent function
%   Detailed explanation goes here

x1 = x(1);
x2 = x(2);
x3 = x(3);
x4 = x(4);

broken = false;

Lc = ((x4-x2)^2+(x3-x1)^2)^0.5; % Calculate the current length of the truss

if round(Lc*10000) == 0 %Check for extremely short edges
    broken = true;
end

% Calculate Bar Force
Fbar = (Ldes-Lc)/Ldes;

l = (x3-x1)/Lc; % Unit vector component for the 1-direction
m = (x4-x2)/Lc; % Unit vector component for the 1-direction

F1 = Fbar*l; % Force component in the 1-direction
F2 = Fbar*m; % Force component in the 2-direction

%
F1x1 = (Ldes*x2^2+Ldes*x4^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x2*x4)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F1x2 = -((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F1x3 = -(Ldes*x2^2+Ldes*x4^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x2*x4)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F1x4 = ((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F2x1 = -((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F2x2 = (Ldes*x1^2+Ldes*x3^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x1*x3)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F2x3 = ((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F2x4 = -(Ldes*x1^2+Ldes*x3^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x1*x3)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F3x1 = -(Ldes*x2^2+Ldes*x4^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x2*x4)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F3x2 = ((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F3x3 = (Ldes*x2^2+Ldes*x4^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x2*x4)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F3x4 = -((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F4x1 = ((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F4x2 = -(Ldes*x1^2+Ldes*x3^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x1*x3)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));
F4x3 = -((x1-x3)*(x2-x4))/((x1-x3)^2+(x2-x4)^2)^(3/2);
F4x4 = (Ldes*x1^2+Ldes*x3^2-((x1-x3)^2+(x2-x4)^2)^(3/2)-2*Ldes*x1*x3)/(Ldes*((x1-x3)^2+(x2-x4)^2)^(3/2));

%Force vector contributions
Ft(1,1) = -F1;
Ft(2,1) = -F2;
Ft(3,1) =  F1;
Ft(4,1) =  F2;

%Consistent tangent contribution
dFdu = [F1x1 F1x2 F1x3 F1x4;
        F2x1 F2x2 F2x3 F2x4;
        F3x1 F3x2 F3x3 F3x4;
        F4x1 F4x2 F4x3 F4x4];
end
