% a_Example_SensitiveHeart SensitiveHeart Examples.
% Copyright (C) 2020 Campbell Bam. See COPYRIGHT.TXT for details.
clear all;  close all; format short; format compact
set(gcf,'renderer','opengl');
fprintf('Mesh on Heart');
% echo on

%% Cb2 FIXED Points on surface
tol = 10^-10;

%% Function assignment
fd_Func = f_SensitiveHeart; FileName = 'Meshdata_2DHeartMesh.mat';


%% Element size distributions
h_max = 0.2;
h_min = .005;
h = @(p) huniform(p,h_max,h_min);% easiest to solve

%% Mesh run
if exist('distmesh2d','file') && ~exist(FileName,'file')
    [p,t]=distmesh2d(@(p) fd_Func.F(p),h,h_max,fd_Func.pbounds,fd_Func.pfix);
    save(FileName,'p','t')
else
    load(FileName)
end

%% Obtain Sensitivities of intial mesh. 
[p1,dXdx1] = MeshSensGen2d(fd_Func,p,t,[],[],tol,[]);
SensitivityPlotter(p1,t,fd_Func,dXdx1,0.4,false);
% 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('% Paused. Press any Key to continue %')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'),pause
close all;

%% Control variable update
ParamUpdate = [2 .4 4 -2 0]';
updatedfd_Func = fd_Func;
updatedfd_Func.Params = fd_Func.Params + ParamUpdate;
X_updates = real(dXdx1*ParamUpdate);
P_updates = [X_updates(1:2:end,1), X_updates(2:2:end,1)];

h = @(p) huniform(p,h_max,h_min);% easiest to solve

[p2,dXdx] = MeshSensGen2d(updatedfd_Func,p1+P_updates,t,[],[],tol);
SensitivityPlotter(p2,t,updatedfd_Func,dXdx,0.4,true);

disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('% Paused. Press any Key to continue %')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'),pause
close all
% Plot in orginal position
figNo = figure;
MESHplotter(p1,t,figNo);
title('Original')

% Overlay in updated predicted position
figNo = figure;
ln = MESHplotter(p1,t,figNo);
ln.EdgeColor = [0.5 0.5 0.5];
ln = MESHplotter(p1+P_updates,t,figNo,true);
ln.EdgeColor = 'k';
ln.LineStyle = ':';
ln.LineWidth = 1;
ln = MESHplotter(p2,t,gcf,true);
ln.EdgeColor = [0.1 0.1 0.1];
ln.LineStyle = '-';
ln.LineWidth = 1;
hold on
xy = updatedfd_Func.plot;
plot(xy(:,1),xy(:,2),'r-.','LineWidth',1.5)
% title('Predicted overlay')
legend('Original','Predicted Mesh','Updated Func. Resolved','Updated Func. Boundary','Location','northeast')


% Overlay in updated predicted position

ParamUpdate = [1 -.1 6 -2 0]';
X_updates = real(dXdx1*ParamUpdate);
P_updates = [X_updates(1:2:end,1), X_updates(2:2:end,1)];
ptmp1 = p1+P_updates;
figNo = figure;
ln = MESHplotter(p1,t,figNo,true);
ln = MESHplotter(ptmp1,t,figNo,true);
ln.EdgeColor = [0.5 0.5 0.5];
ln.LineStyle = ':';
ln.LineWidth = 1;
hold on

ParamUpdate = [-0.5 .5 8 3 0]';
X_updates = real(dXdx1*ParamUpdate);
P_updates = [X_updates(1:2:end,1), X_updates(2:2:end,1)];
ptmp2 = p1+P_updates;
ln = MESHplotter(ptmp2,t,figNo,true);
ln.EdgeColor = [0.5 0.5 0.5];
ln.LineStyle = ':';
ln.LineWidth = 1;

ParamUpdate = [-0.5 -0.5 11 0 .2]';
X_updates = real(dXdx1*ParamUpdate);
P_updates = [X_updates(1:2:end,1), X_updates(2:2:end,1)];
ptmp3 = p1+P_updates;
ln = MESHplotter(p1+P_updates,t,figNo,true);
ln.EdgeColor = [0.5 0.5 0.5];
ln.LineStyle = ':';
ln.LineWidth = 1;

offset = ones(size(p,1),1)*[14 0];
pfinal = p2+offset;
ln = MESHplotter(pfinal,t,figNo,true);
ln.EdgeColor = 'k';
ln.LineStyle = '-';
ln.LineWidth = 1;

pstack = [p1; ptmp1;ptmp2; ptmp3; pfinal];

figNo.OuterPosition = [795   889   982   501];
a = gca;
a.XAxis.Limits = [min(pstack(:,1)) max(pstack(:,1))];
a.XAxis.Visible = 'off';
a.YAxis.Limits = [min(pstack(:,2)) max(pstack(:,2))];
a.YAxis.Visible = 'off';
a.Position = [0.02,0.02,0.96,0.96];


% annotation(figNo,'arrow',[0.2,0.35],[0.55 .67]);
% annotation(figNo,'arrow',[0.198 0.455],[0.47 0.47]);
% annotation(figNo,'arrow',[0.176 0.202],[0.363 0.331]);

annotation(figNo,'arrow',[0.2,0.35],[0.55 .67]);
annotation(figNo,'arrow',[0.48 0.498],[0.686 0.627]);
annotation(figNo,'arrow',[0.498 0.471],[0.313 0.277]);
annotation(figNo,'arrow',[0.443 0.653],[0.184 0.2]);