classdef f_GmshCylinderThroughCylinder
    %FUNCLASS Summary of this class goes here
    %   Detailed explanation goes here
    
    %// Gmsh project created on Sat May 30 15:22:35 2020
    % SetFactory("OpenCASCADE");
    % //+
    % r1 = DefineNumber[ 1, Name "Parameters/radius1" ];
    % //+
    % r2 = DefineNumber[ 1, Name "Parameters/radius2" ];
    % //+
    % L1 = DefineNumber[ 2, Name "Parameters/Length1" ];
    % //+
    % h1 = DefineNumber[ 0.3, Name "Parameters/Height locaiton" ];
    % //+
    % w1 = DefineNumber[ 0.3, Name "Parameters/Width locaiton" ];
    % //+
    % Cylinder(1) = {-1, 0, 0, 2, 0, 0, 1, 2*Pi};
    % //+
    % Cylinder(2) = {-0.5, 0, -2, 1, 0, 4,0.3, 2*Pi};
    % //+
    % BooleanDifference{ Volume{1}; Delete; }{ Volume{2}; Delete; }
    
    properties
        %         L1 R1 R2 Xdisp Ydisp Xoffset
        Params = [2 1, 0.3 0 0 0]';
        L1;
        R1;
        R2;
        Xdisp;
        Ydisp;
        Yoffset;
    end
    
    
    methods
        
        function out = get.L1(obj)
            out = obj.Params(1,1);
            %z
        end
        
        function out = get.R1(obj)
            out = obj.Params(2,1);
            %z
        end
        
        function out = get.R2(obj)
            out = obj.Params(3,1);
            %z
        end
        
        function out = get.Xdisp(obj)
            out = obj.Params(4,1);
            %z
        end
        
        function out = get.Ydisp(obj)
            out = obj.Params(5,1);
            %z
        end
        
        function out = get.Yoffset(obj)
            out = obj.Params(6,1);
            %z
        end
        
    end
    
    properties (Access = private)
        
        Cylinder1 ;
        Cylinder2 ;
    end
    
     methods 
        function out = get.Cylinder1(obj)
            out = [-1, 0, 0, obj.L1, 0, 0, obj.R1];
            %z
        end

        function out = get.Cylinder2(obj)
            out = [-0.5+obj.Xdisp, 0+obj.Ydisp, -2, 1, 0, 4, obj.R2];
            %z
        end
    end
    
    methods (Access = public)
        function pfixout = pfix(obj,p)
            
            pfixout = [];
%             pfixout =[((obj.x1 +1)/obj.Xscale) 0 0
%                 ((obj.x1 - 1)/obj.Xscale) 0 0
%                 0 ((obj.x2 + 1)/obj.Yscale) 0
%                 0 ((obj.x2 - 1)/obj.Yscale) 0
%                 0 0 ((obj.x3 - 2/3)/obj.Zscale)
%                 0 0 ((obj.x3 + 2/3)/obj.Zscale)];
        end
        
        function pboundsout = pbounds(obj)
            
            pboundsout =10*[-1 -1 -1
                           1 1 1];
            
            
        end
        
        function [F] = F(obj,p)
            %Func This returns the distance function
            % This returns signed distance function for a given point
            tol = 0.05;
            
            P1 = obj.Cylinder1(1:3); Vec = obj.Cylinder1(4:6); r = obj.Cylinder1(7);
            cy11 = @(p) obj.Fcylinder(P1,Vec,r,p);
            
            P1 = obj.Cylinder2(1:3); Vec = obj.Cylinder2(4:6); r = obj.Cylinder2(7);
            cy12 = @(p) obj.Fcylinder(P1,Vec,r,p);
            
            F = {};
            
            Fs{1} = @(p) cy11(p);
            Fs{2} = @(p) cy12(p);
            Fs{3} = @(p) obj.FLazyPlane(obj.Cylinder1(1),p);
            Fs{4} = @(p) obj.FLazyPlane(obj.Cylinder1(1)+obj.Cylinder1(4),p);
                       
            F={};
            for i = 1:length(Fs)
                tmpF =Fs{i}(p);
                if abs(tmpF ) < tol
                   F{length(F)+1} = tmpF;
                end
                
            end

        end
        
        function [Grad] = dF(obj,p)
            % dF Summary of this method goes here
            % This returnd the first oFder partial derivitves of the
            % function for a given point
            tol = 0.05;
            
            
            
            P1 = obj.Cylinder1(1:3); Vec = obj.Cylinder1(4:6); r = obj.Cylinder1(7);
            Fs{1} = @(p) obj.Fcylinder(P1,Vec,r,p);
            dFs{1} = @(p) obj.dFcylinder(P1,Vec,r,p);
            
            P1 = obj.Cylinder2(1:3); Vec = obj.Cylinder2(4:6); r = obj.Cylinder2(7);
            Fs{2} = @(p) obj.Fcylinder(P1,Vec,r,p);
            dFs{2} = @(p) obj.dFcylinder(P1,Vec,r,p);
            Grad = {};
            
            Fs{3} = @(p) obj.FLazyPlane(obj.Cylinder1(1),p);
            Fs{4} = @(p) obj.FLazyPlane(obj.Cylinder1(1)+obj.Cylinder1(4),p);
            dFs{3} = @(p) obj.dFLazyPlane(obj.Cylinder1(1),p);
            dFs{4} = @(p) obj.dFLazyPlane(obj.Cylinder1(1)+obj.Cylinder1(4),p);
                       
            
            Grad={};
            for i = 1:length(Fs)
                tmpF =Fs{i}(p);
                if abs(tmpF ) < tol
                   Grad{length(Grad)+1} = dFs{i}(p);
                end
                
            end

        end
%         
        function [SecondGrad] = ddF(obj,p)
            %ddF Second oFder partial derivitives (risidual derivitives)
            %  used for La Grange and CB2 Ellim approx
            tol = 0.05;
            
            P1 = obj.Cylinder1(1:3); Vec = obj.Cylinder1(4:6); r = obj.Cylinder1(7);
            Fs{1} = @(p) obj.Fcylinder(P1,Vec,r,p);
            ddFs{1} = @(p) obj.ddFcylinder(P1,Vec,r,p);
            
            P1 = obj.Cylinder2(1:3); Vec = obj.Cylinder2(4:6); r = obj.Cylinder2(7);
            Fs{2} = @(p) obj.Fcylinder(P1,Vec,r,p);
            ddFs{2} = @(p) obj.ddFcylinder(P1,Vec,r,p);

            
            Fs{3} = @(p) obj.FLazyPlane(obj.Cylinder1(1),p);
            Fs{4} = @(p) obj.FLazyPlane(obj.Cylinder1(1)+obj.Cylinder1(4),p);
            ddFs{3} = @(p) obj.ddFLazyPlane(obj.Cylinder1(1),p);
            ddFs{4} = @(p) obj.ddFLazyPlane(obj.Cylinder1(1)+obj.Cylinder1(4),p);
                       
            SecondGrad={};
            for i = 1:length(Fs)
                tmpF =Fs{i}(p);
                if abs(tmpF ) < tol
                   SecondGrad{length(SecondGrad)+1} = ddFs{i}(p);
                end
            end
        end
        
        function xy = plot(obj)
            %plot Gives boudary plot points
            % This returns signed distance function for a given point
            
%             x(:,1) = linspace(-obj.Xscale,obj.Xscale,1000);
%             y1 = (-(x./obj.Xscale).^2 + 1).^0.5.*obj.Zscale;
%             y2 = -(-(x./obj.Xscale).^2 + 1).^0.5*obj.Zscale;
%             xy =[x, y1;
%                 flip(x), flip(y2)];
            xy = zeros(0,3);
        end

    end
    
    methods (Access = private)
        
        
        function [F] = Fcylinder(obj,P1,Vec,r,p)
            % P1 first end
            % Vec Vector to second end statring at P1
            % r Radius
            % p point
            x0 = p(1,1);
            y0 = p(1,2)-obj.Yoffset;
            z0 = p(1,3);
            
            x1 = P1(1,1);
            x2 = P1(1,1) + Vec(1,1);
            y1 = P1(1,2);
            y2 = P1(1,2) + Vec(1,2);
            z1 = P1(1,3);
            z2 = P1(1,3) + Vec(1,3);

            F = (x1-x0+((x1-x2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2))^2+(y1-y0+((y1-y2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2))^2+(z1-z0+((z1-z2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2))^2-r^2;

        end
        
        function [Grad] = dFcylinder(obj,P1,Vec,r,p)
            x0 = p(1,1);
            y0 = p(1,2)-obj.Yoffset;
            z0 = p(1,3);
            
            x1 = P1(1,1);
            x2 = P1(1,1) + Vec(1,1);
            y1 = P1(1,2);
            y2 = P1(1,2) + Vec(1,2);
            z1 = P1(1,3);
            z2 = P1(1,3) + Vec(1,3);
            
            dFdx0 = 2*((x1-x2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1)*(x1-x0+((x1-x2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2))+(2*(x1-x2)*(y1-y2)*(y1-y0+((y1-y2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(z1-z2)*(z1-z0+((z1-z2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2);
            dFdy0 = 2*((y1-y2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1)*(y1-y0+((y1-y2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2))+(2*(y1-y2)*(z1-z2)*(z1-z0+((z1-z2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)*(x1-x0+((x1-x2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2);
            dFdz0 = 2*((z1-z2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1)*(z1-z0+((z1-z2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2))+(2*(x1-x2)*(z1-z2)*(x1-x0+((x1-x2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(y1-y2)*(z1-z2)*(y1-y0+((y1-y2)*((x0-x1)*(x1-x2)+(y0-y1)*(y1-y2)+(z0-z1)*(z1-z2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2);
            
            Grad = [dFdx0 dFdy0 dFdz0];
            
            ax = find(abs(Grad) < 1e-12);
            Grad(ax) = 0;
        end
                
        function [SecondGrad] = ddFcylinder(obj,P1,Vec,r,p)
            x0 = p(1,1);
            y0 = p(1,2)-obj.Yoffset;
            z0 = p(1,3);
            
            x1 = P1(1,1);
            x2 = P1(1,1) + Vec(1,1);
            y1 = P1(1,2);
            y2 = P1(1,2) + Vec(1,2);
            z1 = P1(1,3);
            z2 = P1(1,3) + Vec(1,3);
            
            ddFdx0dx0  =  2*((x1-x2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1)^2+(2*(x1-x2)^2*(y1-y2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2+(2*(x1-x2)^2*(z1-z2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdx0dy0 = (2*(x1-x2)*(y1-y2)*((x1-x2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)*((y1-y2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)*(z1-z2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdx0dz0 = (2*(x1-x2)*(z1-z2)*((x1-x2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(z1-z2)*((z1-z2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)^2*(z1-z2))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdy0dx0 = (2*(x1-x2)*(y1-y2)*((x1-x2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)*((y1-y2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)*(z1-z2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdy0dy0 = 2*((y1-y2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1)^2+(2*(x1-x2)^2*(y1-y2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2+(2*(y1-y2)^2*(z1-z2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdy0dz0 = (2*(y1-y2)*(z1-z2)*((y1-y2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(y1-y2)*(z1-z2)*((z1-z2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)^2*(y1-y2)*(z1-z2))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdz0dx0 = (2*(x1-x2)*(z1-z2)*((x1-x2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(z1-z2)*((z1-z2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)*(y1-y2)^2*(z1-z2))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdz0dy0 = (2*(y1-y2)*(z1-z2)*((y1-y2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(y1-y2)*(z1-z2)*((z1-z2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)+(2*(x1-x2)^2*(y1-y2)*(z1-z2))/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;
            ddFdz0dz0 = 2*((z1-z2)^2/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)-1)^2+(2*(x1-x2)^2*(z1-z2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2+(2*(y1-y2)^2*(z1-z2)^2)/((x1-x2)^2+(y1-y2)^2+(z1-z2)^2)^2;

            SecondGrad = [ddFdx0dx0 ddFdx0dy0 ddFdx0dz0;
                          ddFdx0dy0 ddFdy0dy0 ddFdy0dz0;
                          ddFdx0dz0 ddFdy0dz0 ddFdz0dz0];
        end
        
        
         function [F] = FLazyPlane(obj,x,p)
              F = p(1,1)-x;
         end
            
         function [grad] = dFLazyPlane(obj,x,p)
             grad = [1 0 0];
         end
            
         function [secondgrad] = ddFLazyPlane(obj,x,p)
             secondgrad = zeros(3,3);
         end
        
        
        
        
        
        
        
function SymCyl(i)
    clear all; format compact
    syms x0 x1 x2 y0 y1 y2 z0 z1 z2 r
    
    t = -((x0 - x1)*(x1 - x2) + (y0 - y1)*(y1 - y2) + (z0 - z1)*(z1 - z2))/((x1 - x2)^2 + (y1 - y2)^2 + (z1 - z2)^2);
    
    F = (x0 - (x1+(x2-x1)*t))^2+...
        (y0 - (y1+(y2-y1)*t))^2+...
        (z0 - (z1+(z2-z1)*t))^2-r^2
    
    dFdx0 = diff(F,x0)
    dFdy0 = diff(F,y0)
    dFdz0 = diff(F,z0)
    
    ddFdx0dx0 = diff(dFdx0,x0)
    ddFdx0dy0 = diff(dFdx0,y0)
    ddFdx0dz0 = diff(dFdx0,z0)
    ddFdy0dx0 = diff(dFdy0,x0)
    ddFdy0dy0 = diff(dFdy0,y0)
    ddFdy0dz0 = diff(dFdy0,z0)
    ddFdz0dx0 = diff(dFdz0,x0)
    ddFdz0dy0 = diff(dFdz0,y0)
    ddFdz0dz0 = diff(dFdz0,z0)
end
        
    end
end

