classdef f_SensitiveHeart
    %FUNCLASS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        Params = [2 2 0 0 1]';
        Height;
        Width;
        Xpos;
        Ypos;
        rad;
    end
    
    methods %Get set properties
        
        function x = get.Width(obj)
            x = obj.Params(1);
        end 
        
        function x = get.Height(obj)
            x = obj.Params(2);
        end 
        
        function x = get.Xpos(obj)
            x = obj.Params(3);
        end
        
        function x = get.Ypos(obj)
            x = obj.Params(4);
        end       
        
        function rad = get.rad(obj)
           rad = obj.Params(5);
        end    
            
    end
    
    methods (Access = public)
        
        function pfixout = pfix(obj)
            pfixout =[obj.Xpos  obj.Ypos+obj.Height*sqrt(obj.rad)
                      obj.Xpos  obj.Ypos-obj.Height*sqrt(obj.rad)
                      obj.Xpos+obj.Width*sqrt(obj.rad)  obj.Ypos
                      obj.Xpos-obj.Width*sqrt(obj.rad)  obj.Ypos];
        end
        
        function pboundsout = pbounds(obj)
            pboundsout =[-2*obj.Width*sqrt(obj.rad)+obj.Xpos -2*obj.Height*sqrt(obj.rad)+obj.Ypos 
                          2*obj.Width*sqrt(obj.rad)+obj.Xpos 2*obj.Height*sqrt(obj.rad)+obj.Ypos];
        end
        
        function [Func] = F(obj,p)
            %Func This returns the distance function
            % This returns signed distance function for a given point
            x = (p(:,1)-obj.Xpos)./obj.Width;
            y = (p(:,2)-obj.Ypos)./obj.Height;
            Func = ((x.^2+y.^2-obj.rad).^3 - x.^2.*y.^3);            
        end
                
        function [Grad] = dF(obj,p)
            %dF First Order derivitive
            %  This returnd the first order partial derivitves of the
            %  function for a given point
            x = (p(:,1)-obj.Xpos)./obj.Width; dx = 1./obj.Width;
            y = (p(:,2)-obj.Ypos)./obj.Height; dy = 1./obj.Height;            
            a = obj.rad;
            
            dFdx = (6*x*(x^2 + y^2 - a)^2 - 2*x*y^3)*dx;
            dFdy = (6*y*(x^2 + y^2 - a)^2 - 3*x^2*y^2)*dy;
            
            Grad = [dFdx dFdy];
        end
        
        function [SecondGrad] = ddF(obj,p)
            %ddF Second order partial derivitives (risidual derivitives)
            %  used for La Grange and CB2 Ellim approx
            
            x = (p(:,1)-obj.Xpos)./obj.Width; dx = 1./obj.Width;
            y = (p(:,2)-obj.Ypos)./obj.Height; dy = 1./obj.Height;
            a = obj.rad;

            ddFdxdx = (24*x^2*(x^2 + y^2 - a) + 6*(x^2 + y^2 - a)^2 - 2*y^3)*dx^2;
            ddFdxdy = (24*x*y*(x^2 + y^2 - a) - 6*x*y^2)*dx*dy;
            ddFdydx = (24*x*y*(x^2 + y^2 - a) - 6*x*y^2)*dx*dy;
            ddFdydy = (24*y^2*(x^2 + y^2 - a) + 6*(x^2 + y^2 - a)^2 - 6*x^2*y)*dy*dy;
            
            SecondGrad = [ddFdxdx ddFdxdy;
                          ddFdydx ddFdydy];
        end
                
        function xy = plot(obj)
            %plot Gives boudary plot points
            % This returns signed distance function for a given point
            bounds = obj.pbounds;
            CurFig = gcf;
            [x,y] = meshgrid(bounds(1,1):.01:bounds(2,1),bounds(1,2):.01:bounds(2,2));
            zz = obj.F([reshape(x,[],1),reshape(y,[],1)]);
            z = reshape(zz,size(x,1),size(x,2));
            tmpfig = figure(100);
            [C,h] = contour(x,y,z,[0 0]);
            Lvls = h.LevelList;
            k1 = 1;
            idxc = find(C(1,:) == Lvls(k1));
            Llen = C(2,idxc);
            
            if (Llen(1) - idxc(1))<10
                idxc = idxc(2:end);
                Llen = C(2,idxc);
            end
            
            xy = C(:,idxc(1)+1 : idxc(1)+1+Llen(1)-1)';
            close(tmpfig)
            try
            figure(CurFig)
            catch 
            end
        end
        
    end
    

end

